#include <RF24.h>
#include "emergency.h"

RF24 _radio(7, 8);                 // nRF24L01 radio on SPI bus + pins 7, 8
const byte _nodeId = 7;            // ID of this node
int _lastMsgSent[MAX_SRC];         // ID of last message sent frpm each source
int _timesSent[MAX_SRC];           // number of times lastMsg has been sent
int _knowledgeMatrix[MAX_SRC][MAX_ID]; // records last message seen by each node

void setup() {
  Serial.begin(115200);
  Serial.println("node start");
  setupRadio(_radio);
  _radio.startListening();
}

void propagateMsg(EmergencyMsg& eMsg) {
  if (eMsg.ttl == 0)
    return;
  /* Prepare message */
  eMsg.ttl--;
  eMsg.senderPath[MAX_TTL-eMsg.ttl-1] = _nodeId;
  memcpy(eMsg.knowledgeVector, _knowledgeMatrix[eMsg.srcId],
         sizeof(int) * MAX_ID);
  /* Wait for channel to be quiet */
  int tries = 0;
  while (_radio.testCarrier()) {        // if other nodes communicating,
    delayMicroseconds(random(10000));   //  wait and try a little later
    _radio.stopListening();             // reset flag for testCarrier()
    _radio.startListening();
    if (tries++ > 20)                   // don't send at all if too busy
      return;
  }
  /* Write out updated message */
  _radio.stopListening();
  _radio.write(&eMsg, sizeof(EmergencyMsg), 1);
  _radio.startListening();
  _timesSent[eMsg.srcId]++;
  /* Print debug info */
  Serial.print("Propagated ");
  eMsg.printMsg();
}

void loop() {
  /* Receive and propagate messages */
  EmergencyMsg eMsg;
  if (_radio.available()) {
    _radio.read(&eMsg, sizeof(EmergencyMsg));
    /* Update knowledge vector */
    _knowledgeMatrix[eMsg.srcId][_nodeId] = eMsg.msgId;
    for (int i = 0; i < MAX_ID; i++) {
      _knowledgeMatrix[eMsg.srcId][i] =
        max(_knowledgeMatrix[eMsg.srcId][i], eMsg.knowledgeVector[i]);
    }
    /* Decide whether or not to propagate */
    if (eMsg.msgId > _lastMsgSent[eMsg.srcId]) {
      _lastMsgSent[eMsg.srcId] = eMsg.msgId;
      _timesSent[eMsg.srcId] = 0;
      propagateMsg(eMsg);   // always propagate new messages
    } else if (eMsg.msgId < _lastMsgSent[eMsg.srcId]) {
      return;               // never propagate old messages
    } else if (random(int(pow(2, _timesSent[eMsg.srcId]))) == 0) {
      propagateMsg(eMsg);   // decrease probability to send each time we send
    }                       //  chance to send: 1, 1/2, 1/4, 1/8, ...
  }
}
