#include "emergency.h"

void setupRadio(RF24 radio) {
  radio.begin();
  radio.setAutoAck(false);
  radio.openWritingPipe(MSG_ADDR);
  radio.openReadingPipe(1, MSG_ADDR);
  radio.setChannel(100);          // 2.500 GHz to avoid Wifi bands
}

EmergencyMsg::EmergencyMsg(int srcId, int msgId) :
  srcId(srcId),
  msgId(msgId),
  senderId(-1),
  ttl(MAX_TTL),
  srcLoc(),
  senderLoc(),
  senderPath() {           // zero initializes locations and path
}

EmergencyMsg::EmergencyMsg() :
  EmergencyMsg(0, 0) {     // zero initializes everything
}

void EmergencyMsg::printMsg() {
  Serial.print("MsgId ");
  Serial.print(this->srcId);
  Serial.print("-");
  Serial.print(this->msgId);
  Serial.print(": ");
  for (int i = 0; i < MAX_TTL; i++) {
    Serial.print(this->senderPath[i]);
    Serial.print(" ");
  }
  Serial.print("ttl: ");
  Serial.println(this->ttl);
}

void EmergencyMsg::serialWriteLoc(byte nodeId) {
  /* Example format: E5 lat long\n */
  if (this->srcId == nodeId)
    Serial.print('S');
  else
    Serial.print('E');
  Serial.print(this->srcId);
  Serial.print(' ');
  Serial.print(this->srcLoc.latitude, 8);
  Serial.print(' ');
  Serial.println(this->srcLoc.longitude, 8);

  if (this->ttl == MAX_TTL)
    return;
  Serial.print('C');
  Serial.print(this->senderId);
  Serial.print(' ');
  Serial.print(this->senderLoc.latitude, 8);
  Serial.print(' ');
  Serial.println(this->senderLoc.longitude, 8);
}
