#ifndef EMERGENCY_H
#define EMERGENCY_H

#include <Arduino.h>
#include <RF24.h>

const int MAX_TTL = 8;             // starting time to live value for messages
const int MAX_ID = 10;             // maximum node ID: range = [0, MAX_ID)
const byte MSG_ADDR[6] = "MNode";  // address to send out emergency messages

struct Location {
  float latitude;
  float longitude;
};

class EmergencyMsg {
public:
  byte ttl;                        // time to live: max 10 hops
  byte srcId;                      // ID of source node that created this msg
  byte senderId;                   // ID of node that sent this message
  int msgId;                       // ID of this message, set by source
  struct Location srcLoc;          // location of the source node
  struct Location senderLoc;       // location of the node that sent this msg
  byte senderPath[MAX_TTL];        // sender IDs in path of message

  EmergencyMsg(int srcId, int msgId);
  EmergencyMsg();
  void printMsg();                 // prints details about message
  void serialWriteLoc(byte nodeId);// write location info out to Serial
};

void setupRadio(RF24 radio);       // setup radio to receive messages

#endif
