// adapted from https://www.pjrc.com/teensy/td_libs_VirtualWire.html

#include <VirtualWire.h>

const int TRANSMIT_PIN = 12;
const int RECEIVE_PIN = 11;
const int TRANSMIT_EN_PIN = 3;

void setup() {
  delay(1000);
  Serial.begin(9600);  // debugging only
  Serial.println("setup receiver");
  /* Initialise the IO and ISR */
  vw_set_tx_pin(TRANSMIT_PIN);
  vw_set_rx_pin(RECEIVE_PIN);
  vw_set_ptt_pin(TRANSMIT_EN_PIN);
  vw_set_ptt_inverted(true);
  vw_setup(2000);      // bits per sec
}

uint8_t recvMessage() {
  uint8_t buf[VW_MAX_MESSAGE_LEN];
  uint8_t buflen = VW_MAX_MESSAGE_LEN;
  /* Wait for message with good checksum */
  while (!vw_get_message(buf, &buflen)) { }
  /* Print message */
  //Serial.print("Receiver got: ");
  //for (int i = 0; i < buflen; i++) {
  //  Serial.print(buf[i], HEX);
  //  Serial.print(' ');
  //}
  //Serial.println();
  return buf[6];       // return the received message ID
}

void sendResponse(uint8_t msgId) {
  //Serial.print("sending ");
  //Serial.println(msgId, HEX);
  char msg[7] = {'r','e','p','l','y',' ','#'};
  msg[6] = msgId;
  vw_send((uint8_t*)msg, 7);
  vw_wait_tx();        // wait until the whole message is gone
}

void loop() {
  vw_rx_start();       // start the receiver running
  uint8_t msgId = recvMessage();
  vw_rx_stop();
  sendResponse(msgId);
}
