// adapted from https://www.pjrc.com/teensy/td_libs_VirtualWire.html

#include <VirtualWire.h>

const int TRANSMIT_PIN = 12;
const int RECEIVE_PIN = 11;
const int TRANSMIT_EN_PIN = 3;
uint8_t _count = 0;

void setup() {
  delay(1000);
  Serial.begin(9600);  // debugging only
  Serial.println("setup transmitter");
  /* Initialise the IO and ISR */
  vw_set_tx_pin(TRANSMIT_PIN);
  vw_set_rx_pin(RECEIVE_PIN);
  vw_set_ptt_pin(TRANSMIT_EN_PIN);
  vw_set_ptt_inverted(true);
  vw_setup(2000);      // bits per sec
}

void sendMessage() {
  Serial.print("sending ");
  Serial.println(_count, HEX);
  char msg[7] = {'h', 'e', 'l', 'l', 'o', ' ', '#'};
  msg[6] = _count;
  vw_send((uint8_t*)msg, 7);
  vw_wait_tx();        // wait until the whole message is gone
  _count += 1;
}

void recvResponse() {
  uint8_t buf[VW_MAX_MESSAGE_LEN];
  uint8_t buflen = VW_MAX_MESSAGE_LEN;
  /* Wait for message with good checksum */
  int timer = 0;
  while (!vw_get_message(buf, &buflen)) {
    timer++;
    delay(1);
    if (timer > 100)
      return;
  }
  /* Print message */
  Serial.print("Sender got: ");
  for (int i = 0; i < buflen; i++) {
    Serial.print(buf[i], HEX);
    Serial.print(' ');
  }
  Serial.print("in ");
  Serial.print(timer);
  Serial.println(" milliseconds.");
}

void loop() {
  sendMessage();
  vw_rx_start();       // start the receiver running
  recvResponse();
  vw_rx_stop();
}
