package com.rkinney4.finalapp;

import android.content.Context;
import android.location.Location;
import com.google.firebase.iid.FirebaseInstanceId;
import com.google.firebase.messaging.FirebaseMessaging;
import android.util.Log;
import android.widget.Toast;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import android.support.v4.content.LocalBroadcastManager;
import android.content.BroadcastReceiver;
import android.content.IntentFilter;
import android.content.Intent;

/**
 * Created by Rachel on 4/21/2017.
 */

public class ClientServerControl implements Control {

    private Context context;
    private MainActivity main;
    String token;
    private final String TAG = "ClientServer";
    private boolean userContinue;
    private boolean emergencyContinue;
    RequestQueue dataQueue;
    private Long totalRTT;
    private Long totalTrips;
    private ArrayList<Long> rttList;


    ClientServerControl(Context context) {
        this.context = context;
        main = (MainActivity) context;
        userContinue = false;
        emergencyContinue = false;
        rttList = new ArrayList<Long>();
        if (dataQueue == null) {
            dataQueue = Volley.newRequestQueue(main);
        }
        LocalBroadcastManager.getInstance(main).registerReceiver((emergencyMessageReceiver), new IntentFilter("Emergency"));
        LocalBroadcastManager.getInstance(main).registerReceiver((mMessageReceiver), new IntentFilter("MyData"));

        // get the firebase token for this phone
        token = FirebaseInstanceId.getInstance().getToken();
        // Log and toast
    }

    @Override
    public void onEmergencyStart() {
        // TODO Remove - for testing
        emergencyContinue = true;
        totalRTT = 0L;
        totalTrips = 0L;
        sendEmergencyPing();
    }


    @Override
    public void onEmergencyStop() {
        emergencyContinue = false;
        if (totalTrips != 0) {
            main.addItem("Average RTT: " + totalRTT / totalTrips, " ", "");
        }
    }

    @Override
    public void onUserStart() {
        userContinue = true;
        sendUserPing();
    }

    @Override
    public void onUserStop() {
        userContinue = false;
    }

    private void sendUserPing() {
        Log.d(TAG, "sending user ping!");
        //this waits for the response from the server, at the moment the value of emergency should always be false
        Response.Listener<JSONObject> responseListener = new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {
                try {
                    JSONObject vehicles = response.getJSONObject("vals");
                    Iterator x = vehicles.keys();
                    JSONArray jsonArray = new JSONArray();

                    while (x.hasNext()){
                        String carId = (String) x.next();
                        JSONArray tmpList = (JSONArray) vehicles.get(carId);
                        double longitude = tmpList.getDouble(0);
                        double latitude = tmpList.getDouble(1);
                        Log.d(TAG, carId + "(" + latitude + "," + longitude + ")");
                        if(!carId.equals(token)) {
                            main.addMapPoint(latitude, longitude, TabFragment.MARKER_OTHER, carId); //TODO update
                        }
                    }
                    main.pruneMapPoints();
                }catch (Exception e) {
                    e.printStackTrace();
                    Log.e(TAG, e.getStackTrace().toString());
                    Log.e(TAG, e.toString());
                    Log.e(TAG, response.toString());
                }
                finally {
                    if(userContinue) sendUserPing();
                }
            }
        };

        Response.ErrorListener errorListener = new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Log.e("sendUserPing", error.toString());
                main.addItem("user timeout?", "", "");
                if (userContinue) sendUserPing();
            }
        };

        //sends the token of the app running and the gps coordinates
        try {
            Location l = main.location;
            for (int i = 0; i < 5; i++) {
                if (l == null) {
                    main.getLocation();
                } else {
                    break;
                }
                l = main.location;
            }
            double lat;
            double lon;
            if (l == null) {
                lat = 0.0;
                lon = 0.0;
            } else {
                lat = l.getLatitude();
                lon = l.getLongitude();
            }
            JsonObjectRequest request = new JsonObjectRequest(
                    Request.Method.POST, "https://cloud2.cnds.jhu.edu/car_loc", (new JSONObject())
                    .put("version", "1.0")
                    .put("id", token)
                    .put("lat", l.getLatitude())
                    .put("lon", l.getLongitude()),
                    responseListener, errorListener);
            dataQueue.add(request);

            main.addMapPoint(lat, lon, TabFragment.MARKER_SELF, token); //TODO update


        }catch(Exception e) {
            Log.e("Exception", String.valueOf(e));
        }
    }

    //Sends an emergency message to FireBase
    private void sendEmergencyPing() {
        totalTrips += 1;
        Log.d(TAG, "sending emergency ping");
        Response.Listener<JSONObject> responseListener = new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {
                try {
                    JSONObject vehicles = response.getJSONObject("vals");
                    Iterator x = vehicles.keys();
                    JSONArray jsonArray = new JSONArray();

                    while (x.hasNext()){
                        String carId = (String) x.next();
                        JSONArray tmpList = (JSONArray) vehicles.get(carId);
                        double longitude = tmpList.getDouble(0);
                        double latitude = tmpList.getDouble(1);
                        Log.d(TAG, carId + "(" + latitude + "," + longitude + ")");
                        if(!carId.equals(token)) {
                            main.addMapPoint(latitude, longitude, TabFragment.MARKER_OTHER, carId); //TODO update
                        }
                    }
                    main.pruneMapPoints();

                } catch (Exception e) {
                    Log.e(TAG, response.toString());
                }
                finally {
                    if(emergencyContinue) sendEmergencyPing();
                }
            }
        };

        Response.ErrorListener errorListener = new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Log.e("sendDataPoint", error.toString());
                main.addItem("emer timeout?", "", "");
                if (emergencyContinue) sendEmergencyPing();
            }
        };

        //send to JSON object to server
        try {
            Location l = main.location;
            for (int i = 0; i < 5; i++) {
                if (l == null) {
                    main.getLocation();
                } else {
                    break;
                }
                l = main.location;
            }
            double lat;
            double lon;
            if (l == null) {
                lat = 0.0;
                lon = 0.0;
            } else {
                lat = l.getLatitude();
                lon = l.getLongitude();
            }
            JsonObjectRequest request = new JsonObjectRequest(
                    Request.Method.POST, "https://cloud2.cnds.jhu.edu/emergency", (new JSONObject())
                    .put("version", "1.0")
                    .put("id", token)
                    .put("lat", lat)
                    .put("lon", lon),
                    responseListener, errorListener);
            main.addMapPoint(lat, lon, TabFragment.MARKER_SELF, token); //TODO update
            // add current time
            rttList.add(0, System.currentTimeMillis());
            dataQueue.add(request);
        }catch(Exception e) {
            Log.e("Error in first sent", String.valueOf(e));
        }
    }

    /**Listens for a message from FireBase and makes a toast
     * */
    private BroadcastReceiver emergencyMessageReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Long rttStop = System.currentTimeMillis();
            Log.d(TAG, "emergency!");
            String e_id = intent.getStringExtra("e_id");
            double lat = intent.getDoubleExtra("lat", 0.0);
            double lon = intent.getDoubleExtra("lon", 0.0);
            if(!e_id.equals(token)) {
                main.addMapPoint(lat, lon, TabFragment.MARKER_EMERGENCY, e_id); //TODO update
                main.addItem("Emergency!", "", "");
            } else {
                if (rttList.size() > 0) {
                    Long rttStart = rttList.get(0);
                    Long diff = rttStop - rttStart;
                    main.addItem("RTT: " + (rttStop - rttStart), "", "");
                    totalRTT += diff;
                    totalTrips += 1;
                }
            }
        }
    };

    /**Listens for a message from FireBase and makes a toast
     * */
    private BroadcastReceiver mMessageReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Log.d(TAG, "ping reccieved!");
        }
    };

}
