package com.rkinney4.finalapp;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.TabItem;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.content.SharedPreferences;
import android.net.Uri;
import android.support.design.widget.TabLayout;
import android.support.v4.view.ViewPager;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.Toast;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

public class MainActivity extends AppCompatActivity implements TabFragment.OnFragmentInteractionListener,
        GoogleApiClient.OnConnectionFailedListener,
        GoogleApiClient.ConnectionCallbacks,
        LocationListener {

    ViewPager viewPager;
    TabAdapter tabAdapter;

    boolean active = false;
    boolean setup_location = false;
    ClientServerControl clientServerControl;
    RFControl rfControl;

    // Location Services
    static final int REQUEST_LOCATION = 1;
    GoogleApiClient gApiClient;
    LocationRequest locReq;
    boolean noLocation;
    Location location;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar myToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(myToolbar);

        // Set up tab paging
        viewPager = (ViewPager) findViewById(R.id.viewpager);
        tabAdapter = new TabAdapter(getSupportFragmentManager(), this);
        viewPager.setAdapter(tabAdapter);
        final TabLayout tabLayout = (TabLayout) findViewById(R.id.tablayout);
        tabLayout.setupWithViewPager(viewPager);

        // Prevent tab changes while sending / recieving
        for (int i = 0; i < 2; i++) {
            View tab = ((LinearLayout)tabLayout.getChildAt(0)).getChildAt(i);
            tab.setTag(i);
            tab.setOnTouchListener(new View.OnTouchListener(){
                public boolean onTouch(View v, MotionEvent event) {
                    int action = event.getAction();

                    if(action == MotionEvent.ACTION_UP)
                    {
                        int currentTab = tabLayout.getSelectedTabPosition();
                        int selectedTab = (int) v.getTag();

                        if( currentTab != selectedTab && active)
                        {
                            Toast.makeText(v.getContext(), getString(R.string.toast_tab_change), Toast.LENGTH_LONG).show();
                            return true;
                        }
                    }
                    return false;
                }
            });
        }

        // Handle Location API
        gApiClient = new GoogleApiClient.Builder(this)
                .enableAutoManage(this, this)
                .addApi(LocationServices.API)
                .addConnectionCallbacks(this)
                .addOnConnectionFailedListener(this)
                .build();
        locReq = LocationRequest.create()
                .setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY)
                .setInterval(1000)        //   1 second, in milliseconds
                .setFastestInterval(250); // 1/4 second, in milliseconds
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (gApiClient.isConnected()) {
            LocationServices.FusedLocationApi.removeLocationUpdates(gApiClient, this);
        }
        setup_location = false;
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.toolbar, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_settings:
                Intent intent = new Intent(this, Settings.class);
                //EditText editText = (EditText) findViewById(R.id.editText);
                //String message = editText.getText().toString();
                //intent.putExtra(EXTRA_MESSAGE, message);
                startActivity(intent);
                break;
            default:
                break;
        }
        return true;
    }

    @Override
    public void onFragmentInteraction(Uri uri) {

    }

    public int getMode() {
        return ((TabFragment)tabAdapter.getItem(viewPager.getCurrentItem())).client_type;

    }
    /*********** Connection Functions ***********/

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {
        /* Handle Google Play Services Connection Failure */
        // May need to add handling here if problem is encountered
        Toast.makeText(this, R.string.play_serv_error,Toast.LENGTH_LONG).show();
        noLocation = true;
        Log.w("onConnectionFailed", "acceptingNoLocation - Google Play, Err: " + connectionResult.getErrorMessage());
    }

    @Override
    public void onConnected(@Nullable Bundle bundle) {
        Log.i("onConnected", "Google Play - CONNECTION SUCCESSFUL");
    }

    @Override
    public void onConnectionSuspended(int i) {
        Log.i("onConnectionSuspended", "Reason: " + ((i == CAUSE_NETWORK_LOST) ? "Network Lost" : "Service Disconnected"));
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == REQUEST_LOCATION ) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                noLocation = true;
                Log.w("onReqPermissResult", "Location Permission Denied, acceptingNoLocation = true");
            } else {
                Log.i("onReqPermissResult", "Location Permissions Granted");
            }
        }
    }

    /********** Application Functions **********/

    // Adds an item to the log of the current tab
    public void addItem(String time, String type, String message) {
        ((TabFragment)tabAdapter.getItem(viewPager.getCurrentItem())).addItem(time, type, message);
    }

    public void addMapPoint(double lat, double lng, int type, String id) {
        LatLng l = new LatLng(lat, lng);
        ((TabFragment)tabAdapter.getItem(viewPager.getCurrentItem())).addMapPoint(l, type, id);
    }

    public void pruneMapPoints() {
        ((TabFragment) tabAdapter.getItem(viewPager.getCurrentItem())).pruneMapPoints();
    }

    @Override
    public void onLocationChanged(Location newLocation) {
        location = newLocation;
    }

    // 0 = Success, -1 = Failure
    public int pullLocation() {
        if (!noLocation) {
            try {
                LocationServices.FusedLocationApi.requestLocationUpdates(gApiClient, locReq, this);
                Location loc = LocationServices.FusedLocationApi.getLastLocation(gApiClient);
                if (loc == null) {
                    Log.i("pullLocation", "Location null, requesting updates");
                    //LocationServices.FusedLocationApi.requestLocationUpdates(gApiClient, locReq, this);
                } else {
                    setup_location = true;
                    onLocationChanged(loc);
                }
            } catch (SecurityException e) {
                ActivityCompat.requestPermissions(this,
                        new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION},
                        REQUEST_LOCATION);
                location = null;
                return -1;
            } catch (IllegalStateException e) {
                Log.w("pullLocation", "onStop probably called. Error: " + e.getMessage());
                return -1;
            }
        } else {
            location = null;
        }
        return 0;
    }

    public Location getLocation() {
        pullLocation();
        return location;
    }

    // ADJUST THIS FUNCTION IF NECESSARY
    public void setupClientServer() {
        clientServerControl = new ClientServerControl(this);
    }

    // ADJUST THIS FUNCTION IF NECESSARY
    public void setupRF() {
        rfControl = new RFControl(this);
    }
}

