package com.rkinney4.finalapp;

import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbManager;
import android.location.Location;
import android.util.Log;

import com.felhr.usbserial.UsbSerialDevice;
import com.felhr.usbserial.UsbSerialInterface;

import java.nio.ByteBuffer;

import java.io.UnsupportedEncodingException;
import java.nio.ByteOrder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutionException;

import android.os.Handler;


import static android.content.Context.USB_SERVICE;

/**
 * Created by Rachel on 4/21/2017.
 */

/* Much code borrowed from:
   https://www.allaboutcircuits.com/projects/communicate-with-your-arduino-through-android/ */

public class RFControl implements Control {

    private static final String ACTION_USB_PERMISSION = "com.rkinney4.finalapp.USB_PERMISSION";

    private Context context;
    private MainActivity main;

    UsbManager usbManager;
    UsbDevice device;
    UsbSerialDevice serialPort;
    UsbDeviceConnection connection;
    String message = "";
    private Handler myHandler;


    private int mInterval = 2000; // 2 seconds by default, can be changed later
    private int mapUpdateInterval = 1000;
    private final BroadcastReceiver broadcastReceiver;

    private ArrayList<LocationDatapoint> pointList;

    ByteBuffer sent;
    ByteBuffer received = ByteBuffer.allocate(19);

    RFControl(Context context) {
        this.context = context;
        main = (MainActivity) context;
        pointList = new ArrayList<LocationDatapoint>();

        // Defining a Callback which triggers whenever data is read.
        final UsbSerialInterface.UsbReadCallback mCallback = new UsbSerialInterface.UsbReadCallback() {
            @Override
            public void onReceivedData(byte[] arg0) {
                try {
                    message += new String(arg0, "UTF-8");

//                    main.addItem("Total message now: " , message, "");
                    while(message.indexOf("\n") != -1) {
                        String toParse = message.substring(0, message.indexOf("\n"));
                        message = message.substring(message.indexOf("\n") + 1);
                        parseVehicles(toParse);
                    }

                }catch(Exception e) {
//                    main.addItem("Bad: ", e.toString(), " SWAG");
                }

            }
        };

        //Broadcast Receiver to automatically start and stop the Serial connection.
        broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                try {
                    if (intent.getAction().equals(ACTION_USB_PERMISSION)) {
                        boolean granted = intent.getExtras().getBoolean(UsbManager.EXTRA_PERMISSION_GRANTED);
                        if (granted) {
                            connection = usbManager.openDevice(device);
                            serialPort = UsbSerialDevice.createUsbSerialDevice(device, connection);
                            if (serialPort != null) {
                                if (serialPort.open()) { //Set Serial Connection Parameters.
                                    //setUiEnabled(true); TODO Remove
                                    serialPort.setBaudRate(9600);
                                    serialPort.setDataBits(UsbSerialInterface.DATA_BITS_8);
                                    serialPort.setStopBits(UsbSerialInterface.STOP_BITS_1);
                                    serialPort.setParity(UsbSerialInterface.PARITY_NONE);
                                    //serialPort.setFlowControl(UsbSerialInterface.FLOW_CONTROL_OFF);


                                    try {
                                        serialPort.read(mCallback);
                                    } catch (Exception e) {
                                        main.addItem("Reading threw as exception ", "" + e.toString(), "");
                                    }
                                } else {
                                    Log.d("SERIAL", "PORT NOT OPEN");
                                    main.addItem("Debug: ", "not open ", "");

                                }
                            } else {
                                Log.d("SERIAL", "PORT IS NULL");
                                main.addItem("Debug: ", "null ", "");

                            }
                        } else {
                            Log.d("SERIAL", "PERM NOT GRANTED");
                            main.addItem("Debug: ", "not granted", "");

                        }
                    } else if (intent.getAction().equals(UsbManager.ACTION_USB_DEVICE_ATTACHED)) {
                        //onClickStart(startButton); TODO Change
                        main.addItem("Debug: ", "Attached", "");
                        setUp();


                    } else if (intent.getAction().equals(UsbManager.ACTION_USB_DEVICE_DETACHED)) {
                        //onClickStop(stopButton); TODO Change
                    }
                } catch (Exception e) {
                    main.addItem("crash in setup!", e.toString(), "");
                }
            };
        };

        try {
            usbManager = (UsbManager) context.getSystemService(USB_SERVICE);
            //setUiEnabled(false); TODO Remove
            IntentFilter filter = new IntentFilter();
            filter.addAction(ACTION_USB_PERMISSION);
            filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
            filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
            context.registerReceiver(broadcastReceiver, filter);
            setUp();

            myHandler = new Handler();
            sendGPS.run();
            updateMap.run();
        }catch(Exception e) {
//            main.addItem("Problem in constructor ", "" + e.toString(), "");
        }
    }




    public void setUp() {


        try {
            HashMap usbDevices = usbManager.getDeviceList();
            if (!usbDevices.isEmpty()) {
                boolean keep = true;
                //for (int i : usbDevices.entrySet()) {
                for (Object entry : usbDevices.entrySet()) {
                    device = (UsbDevice) ((Map.Entry) entry).getValue();
                    int deviceVID = device.getVendorId();

                    if (deviceVID == 0x2A03)//Arduino Vendor ID
                    {

                        PendingIntent pi = PendingIntent.getBroadcast(context, 0,
                                new Intent(ACTION_USB_PERMISSION), 0);
                        usbManager.requestPermission(device, pi);
                        keep = false; //TODO does this ignore a device if it's the first time and requesting permissions?

                    } else {
                        connection = null;
                        device = null;
                    }

                    if (!keep)
                        break;
                }
            }//end if
        }catch(Exception e) {
//            main.addItem("Exception in setup ", "" + e.toString(), "");
        }
    }

    Runnable sendGPS = new Runnable() {
        @Override
        public void run() {
//            if (main.getMode() != 1) {
//                myHandler.postDelayed(sendGPS, mInterval);
//            } else {
                main.getLocation();
                Location l = main.location;
                for (int i = 0; i < 5; i++) {
                    if (l == null) {
                        main.getLocation();
                    } else {
                        break;
                    }
                    l = main.location;
                }
                double lat;
                double lon;
                if (l == null) {
                    lat = 0;
                    lon = 0;
                } else {
                    lat = l.getLatitude();
                    lon = l.getLongitude();
                }
//                main.addItem("Sending GPS: ", "" + lat, " " + lon);
                try {
                    serialPort.write(("" + lat + "," + lon + ";").getBytes());
                    main.addMapPoint(lat, lon, TabFragment.MARKER_SELF, "");
                } catch (Exception e) {
//                    main.addItem("Debug", " failed to send GPS " + e.toString(), "");
                } finally {
                    myHandler.postDelayed(sendGPS, mInterval);
                }
  //          }
        }
    };

    Runnable updateMap = new Runnable() {
        @Override
        public void run() {
            // Periodically update map with the points
            // Necessary because this must happen in the main thread
//            if (main.getMode() != 1) {
//                myHandler.postDelayed(updateMap, mInterval);

//            } else {
                try {
//                    main.addItem("Updating map", "", "");
                    for (LocationDatapoint point : pointList) {
                        main.addMapPoint(point.latitude, point.longitude, point.type, point.id);
                    }
                    pointList.clear();
                    main.pruneMapPoints();
                } catch (Exception e) {
//                    main.addItem("Debug", " Error in updateMap(): " + e.toString(), "");
                } finally {
                    myHandler.postDelayed(updateMap, mapUpdateInterval);
                }
            //}
        }
    };






    @Override
    public void onEmergencyStart() {
        try {
            serialPort.write("E".getBytes());
        }catch(Exception e) {
            main.addItem("Debug", e.toString(), "");

        }

    }

    @Override
    public void onEmergencyStop() {
        try {
            serialPort.write("N".getBytes());
        }catch(Exception e) {
            main.addItem("Debug", e.toString(), "");
        }
    }

    @Override
    public void onUserStart() {
        try {
            serialPort.write("C".getBytes());
        }catch(Exception e) {
            main.addItem("Debug", e.toString(), "SWAG");

        }
    }

    @Override
    public void onUserStop() {
        try {
            serialPort.write("N".getBytes());
        }catch(Exception e) {
            main.addItem("Debug", e.toString(), "");

        }

    }

    public void parseVehicles(String message) {

        // EV       SENDER
        // iaaaaooooiaaaaoooo

        // 012345678901234567

        //pass this info  to display
//        main.addItem("Inside parse vehicles", " ", " ");
        try {
            String[] fields = message.split(" ");
            String id = fields[0]; // cut out the E/S/C char
            Double lat = Double.parseDouble(fields[1]);
            Double lon = Double.parseDouble(fields[2]);
            int type = TabFragment.MARKER_SELF;
            if (id.contains("E")) {
                type = TabFragment.MARKER_EMERGENCY;
            } else if (id.contains("C")) {
                type = TabFragment.MARKER_OTHER;
            }
            id = id.substring(1);
            pointList.add(new LocationDatapoint(lat, lon, type, id));
            main.addItem("ID: " + id, " Lat: " + lat, " Lon: " + lon);
        } catch (Exception e) {
//            main.addItem("broken", "", "");
        }
    }

    private float handleData(byte[] mes, int srcStart, int destStart, int size) {

        float toReturn;
        byte[] temp = new byte[4];
        ByteBuffer b = ByteBuffer.allocate(4);
        System.arraycopy(mes, srcStart, temp, destStart, size);
        b = ByteBuffer.allocate(4);
        b.order(ByteOrder.BIG_ENDIAN);
        b.put(temp);
        toReturn = b.getFloat();
        return toReturn;
    }
}
