package com.rkinney4.finalapp;

import android.content.Context;
import android.content.SharedPreferences;
import android.location.Location;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;


/**
 * A simple {@link Fragment} subclass.
 * Activities that contain this fragment must implement the
 * {@link TabFragment.OnFragmentInteractionListener} interface
 * to handle interaction events.
 * Use the {@link TabFragment#newInstance} factory method to
 * create an instance of this fragment.
 */
public class TabFragment extends Fragment implements OnMapReadyCallback {

    public static final int EMER = 0;
    public static final int USER = 1;

    public static final int CLISRV_HYB = 0;
    public static final int RF = 1;

    public static final int MARKER_SELF = 0;
    public static final int MARKER_OTHER = 1;
    public static final int MARKER_EMERGENCY = 2;

    public static final int MAX_MARKERS = 10;

    public static final long TIME_OUT = 10000000000L;

    private Context context;
    private MainActivity mainactivity;
    private SharedPreferences prefs;

    // UI Members
    private View root;
    private ListView log;
    private Button send_button;
    private int img_vehicle_id;

    private GoogleMap map;
    private LinkedHashMap<String,Marker> markers;
    private HashMap<String, Long> timestamps;


    private Marker self;

    public int client_type;
    public int mode;
    private LogAdapter adapter;


    // Other
    private OnFragmentInteractionListener mListener;

    public TabFragment() {
        // Required empty public constructor
    }

    public static TabFragment newInstance(int position, Context c) {
        TabFragment fragment = new TabFragment();

        // Handle Args/Mode
        Bundle args = new Bundle();
        args.putInt(c.getString(R.string.ARG_MODE), position);
        fragment.setArguments(args);

        return fragment;
    }

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        client_type = getArguments().getInt(getString(R.string.ARG_MODE));
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        root = inflater.inflate(R.layout.fragment_tab, container, false);

        //Shared Preferences
        prefs = context.getSharedPreferences(getString(R.string.prefs), Settings.MODE_PRIVATE);

        // Handle UI
        setVehicleImage();

        // Set up Log ListView
        log = (ListView) root.findViewById(R.id.log);
        adapter = new LogAdapter(context);
        log.setAdapter(adapter);

        // Handle Clear
        root.findViewById(R.id.button_clear)
                .setOnClickListener(new View.OnClickListener() {
                                        @Override
                                        public void onClick(View view) {
                        adapter.clear();
                        adapter.notifyDataSetChanged();
                        clearMapPoints();
                        }
                    }
                );

        /* Handle Map */
        // Get the SupportMapFragment and request notification
        // when the map is ready to be used.
        SupportMapFragment mapFragment = (SupportMapFragment) getChildFragmentManager()
                .findFragmentById(R.id.map);
        mapFragment.getMapAsync(this);
        markers = new LinkedHashMap<>();
        timestamps = new HashMap<>();

        /* Handle Client-type specific Controls */
        send_button = (Button) root.findViewById(R.id.button_send);
        if (client_type == EMER) {
            send_button.setText(R.string.sending_start);
        } else {
            send_button.setText(R.string.receive_start);
        }
        send_button.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!mainactivity.setup_location && (mainactivity.pullLocation() == -1)) {
                    Log.i("sendOnClick", "Setting Up Location?");
                    return;
                }

                if ((mode == CLISRV_HYB) && mainactivity.clientServerControl == null) {
                    mainactivity.setupClientServer();
                } else if ((mode == RF) && mainactivity.rfControl == null) {
                    mainactivity.setupRF();
                } else {
                    Log.d("TabFragmentModeHandle", "HIT ELSE STATEMENT: MODE: " + mode);
                }

                mainactivity.active = !mainactivity.active;

                Log.i("sendOnClick", "" + (mode * 100 + client_type * 10 + (mainactivity.active ? 1 : 0)));
                switch (mode * 100 + client_type * 10 + (mainactivity.active ? 0 : 1)) {
                    case 0: send_button.setText(getString(R.string.sending_stop));
                            mainactivity.clientServerControl.onEmergencyStart(); break;
                    case 1: send_button.setText(getString(R.string.sending_start));
                            mainactivity.clientServerControl.onEmergencyStop(); break;
                    case 10: send_button.setText(getString(R.string.receive_stop));
                             mainactivity.clientServerControl.onUserStart(); break;
                    case 11: send_button.setText(getString(R.string.receive_start));
                             mainactivity.clientServerControl.onUserStop(); break;
                    case 100: send_button.setText(getString(R.string.sending_stop));
                              mainactivity.rfControl.onEmergencyStart(); break;
                    case 101: send_button.setText(getString(R.string.sending_start));
                              mainactivity.rfControl.onEmergencyStop(); break;
                    case 110: send_button.setText(getString(R.string.receive_stop));
                              mainactivity.rfControl.onUserStart(); break;
                    case 111: send_button.setText(getString(R.string.receive_start));
                              mainactivity.rfControl.onUserStop(); break;
                    default:
                        Log.e("sendOnClick", "Unknown mode/type combination: " + mode + " " + client_type);
                }
            }
        });

        /* Handle Mode specific Controls */
        mode = prefs.getInt(getString(R.string.pref_mode), 0);
        return root;
    }

    @Override
    public void onResume() {
        super.onResume();
        setVehicleImage();

        mode = prefs.getInt(getString(R.string.pref_mode), 0);
        ((TextView) root.findViewById(R.id.current_mode)).setText(context.getResources().getStringArray(R.array.set_modes)[mode]);
    }

    @Override
    public void onAttach(Context context) {
        super.onAttach(context);
        if (context instanceof OnFragmentInteractionListener) {
            mListener = (OnFragmentInteractionListener) context;
        } else {
            throw new RuntimeException(context.toString()
                    + " must implement OnFragmentInteractionListener");
        }
        this.context = context;
        mainactivity = (MainActivity) context;
    }

    @Override
    public void onDetach() {
        super.onDetach();
        mListener = null;
    }

    @Override
    public void onMapReady(GoogleMap googleMap) {
        map = googleMap;
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     * <p>
     * See the Android Training lesson <a href=
     * "http://developer.android.com/training/basics/fragments/communicating.html"
     * >Communicating with Other Fragments</a> for more information.
     */
    public interface OnFragmentInteractionListener {
        // TODO: Update argument type and name
        void onFragmentInteraction(Uri uri);
    }

    /**************** Helper Functions ****************/

    public void addItem(String time, String type, String message) {
        adapter.addItem(time, type, message);
        adapter.notifyDataSetChanged();
    }

    public void addMapPoint(LatLng coord, int type, String id) {
        if (markers.containsKey(id)) {
            markers.get(id).remove();
            markers.remove(id);
            timestamps.remove(id);
        }

        if (markers.size() == MAX_MARKERS) {
            Iterator<Marker> iter = markers.values().iterator();
            Marker m = null;
            while (iter.hasNext()) {
                m = iter.next();
            }
            if (m != null) {
                markers.remove(m.getTitle());
                m.remove();
                timestamps = new HashMap<>();
            }
        }

        Marker m;
        if (type == MARKER_OTHER)
            m = map.addMarker(new MarkerOptions().position(coord)
                .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_BLUE))
                .title(id));
        else if (type == MARKER_EMERGENCY) {
            m = map.addMarker(new MarkerOptions().position(coord).icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_RED))
                    .title(id));
        }
        else {
            m = map.addMarker(new MarkerOptions().position(coord)
                .icon(BitmapDescriptorFactory.defaultMarker(BitmapDescriptorFactory.HUE_YELLOW))
                .title(id));
            self = m;
        }
        markers.put(id, m);
        timestamps.put(id, System.nanoTime()  + TIME_OUT);
        //if(markers.size() == 0) {
            Location l = mainactivity.getLocation();
            if (l != null) {
                map.moveCamera(CameraUpdateFactory.newLatLng(new LatLng(l.getLatitude(), l.getLongitude())));
            }
            map.moveCamera(CameraUpdateFactory.zoomTo(16));
        //}
    }

    public void pruneMapPoints() {
        Iterator<Map.Entry<String, Marker>> iter = markers.entrySet().iterator();
        LinkedList<String> toRemove = new LinkedList<>();
        while (iter.hasNext()) {
            String id = iter.next().getKey();
            if(timestamps.get(id) < System.nanoTime()) {
                toRemove.add(id);

            }
            else
                break;
            Log.d("Pruning map", "Tried to prune map");
        }
        for(String id : toRemove) {
            markers.get(id).remove();
            markers.remove(id);
            timestamps.remove(id);
        }

    }

    private void clearMapPoints() {
        Iterator<Marker> iter = markers.values().iterator();
        Marker m = null;
        while (iter.hasNext()) {
            m = iter.next();
            markers.remove(m.getTitle());
        }
        markers.clear();
        timestamps.clear();
    }

    private void setVehicleImage() {
        switch (client_type) {
            case EMER:
                switch (prefs.getInt(getString(R.string.pref_vehicle), 0)) {
                    case 0: img_vehicle_id = R.drawable.ic_ambulance; break;
                    case 1: img_vehicle_id = R.drawable.ic_firetruck; break;
                    case 2: img_vehicle_id = R.drawable.ic_policecar; break;
                    default:
                        Log.e("Tab " + client_type + " onCreate", "Unknown vehicle");
                        img_vehicle_id = R.drawable.ic_ambulance;
                }
                break;
            case USER:
                img_vehicle_id = R.drawable.ic_car;
                break;
            default:
                Log.e("Tab " + client_type + " onCreate", "Unknown client_type");
        }
        ((ImageView) root.findViewById(R.id.img_vehicle)).setImageResource(img_vehicle_id);
    }
}