package com.rkinney4.httptest;

import android.location.Location;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.Serializable;

/**
 * Created by Rachel on 2/24/2017.
 */

class DataPoint implements Serializable {
    enum Field {TYPE, URL, RTT, TIME, ID, NETCLASS, SPEED}

    static long count = 0;

    private final long id;
    private final int type;
    private final String url, netClass, carrier, uuid;
    private final long rtt;
    private final long time;
    private final Float speed;
    private final Float bearing;
    private final Double latitude;
    private final Double longitude;
    final boolean manualSpeedCalc;

    public DataPoint(int type, String url, long rtt, long time, String netClass, String carrier, Location location, String uuid, boolean manualSpeedCalc) {
        this.id = ++count;
        this.type = type;
        this.url = url;
        this.rtt = rtt;
        this.time = time;
        this.netClass = netClass;
        this.carrier = carrier;
        this.uuid = uuid;
        this.manualSpeedCalc = manualSpeedCalc;

        // Handle location
        if (location == null) {
            Log.i("new DataPoint", "Location null");
            speed = bearing = null;
            latitude = longitude = null;
        } else {
            latitude = location.getLatitude();
            longitude = location.getLongitude();
            speed = (location.hasSpeed()) ? location.getSpeed() : null;
            bearing = (location.hasBearing()) ? location.getBearing() : null;
            Log.i("new DataPoint", "S: " + speed + " B: " + bearing + " Lat: " + latitude + "Long: " + longitude);
        }
    }

    public float getNumField(Field f) {
        switch (f) {
            case RTT : return rtt;
            case TIME : return time;
            case ID : return id;
            case SPEED : return speed;
            default:
                Log.e("getNumField", "Field not numeric");
                return -1;
        }
    }

    public String fieldToString(Field f) {
        switch (f) {
            case TYPE :
                switch (type) {
                    case MainActivity.HTTP : return "HTTP";
                    default:
                        Log.e("fieldToString", "Unknown Type");
                        return "UNKNOWN";
                }
            case URL : return url;
            case RTT : return Long.toString(rtt);
            case TIME : return Long.toString(time);
            case ID : return Long.toString(id);
            case NETCLASS : return netClass;
            case SPEED : return Double.toString(speed);
            default:
                Log.e("fieldToString", "Unknown Field");
                return "UNKNOWN";
        }
    }

    public JSONObject asJsonObject() {
        try {
            return (new JSONObject())
                    .put("version", "1.0") // So server can handle outdated clients
                    .put("type", fieldToString(Field.TYPE))
                    .put("url", url)
                    .put("rtt", rtt)
                    .put("netclass", netClass)
                    .put("latitude", latitude)
                    .put("longitude", longitude)
                    .put("bearing", bearing)
                    .put("speed", speed);
        } catch (JSONException e) {
            Log.e("asJsonObject", String.valueOf(e));
            return null;
        }
    }
}
