package com.rkinney4.httptest;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.location.Location;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.ActivityCompat;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.telephony.TelephonyManager;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.webkit.URLUtil;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.android.volley.AuthFailureError;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.github.mikephil.charting.charts.CombinedChart;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.location.LocationServices;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.toolbox.JsonObjectRequest;
import com.android.volley.toolbox.Volley;

import org.json.JSONObject;

public class MainActivity extends AppCompatActivity implements GoogleApiClient.OnConnectionFailedListener,
        GoogleApiClient.ConnectionCallbacks,
        LocationListener {

    private static final String SERVER_DATA_ENDPOINT = "http://10.0.2.2:8888/data";

    // Test Type
    static final int HTTP = 0;

    // Modes
    static final int MANUAL = 0;
    static final int CONTINUOUS = 1;
    static final int FINITE = 2;

    // Android / UI Objects
    Context context;
    private SharedPreferences prefs;
    private String uniqueID;
    Spinner mode_spinner, series_spinner, xaxis_spinner, yaxis_spinner;
    Button send, stop, reset;
    EditText editUrl, editNumReqs;
    TextView numResp, meanTime, medTime;

    // Location Services
    static final int REQUEST_LOCATION = 1;
    static final int SPEED_MAX_INTERVAL = 2000;
    static final int SPEED_MIN_INTERVAL = 250;
    GoogleApiClient gApiClient;
    LocationRequest locReq;
    Location location, prevMaxLocation, prevMinLocation;
    boolean confirmSend;
    boolean acceptNoLocation;
    boolean manSpeedCalc;

    // Connection Variables
    TelephonyManager teleMgr;
    ConnectivityManager connMgr;
    SendHTTP sendHTTP;
    String url, netClass, carrier;
    int numReqs;
    int mode;
    int type = HTTP;

    ArrayList<Long> times;
    Chart chart;
    Long timeSent;

    // data collection vars
    private static RequestQueue dataQueue;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar myToolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(myToolbar);
        context = this;
        prefs = getPreferences(MODE_PRIVATE);
        confirmSend = false;
        acceptNoLocation = false;

        // Get / Create Unique ID
        if (!prefs.contains(getString(R.string.pref_UUID))) {
            uniqueID = UUID.randomUUID().toString();
            SharedPreferences.Editor edit = prefs.edit();
            edit.putString(getString(R.string.pref_UUID), uniqueID);
            edit.apply();
        } else {
            uniqueID = prefs.getString(getString(R.string.pref_UUID), "ERROR");
        }


        times = new ArrayList<>();
        if (dataQueue == null)
            dataQueue = Volley.newRequestQueue(context);

        // Create UI
        editNumReqs = (EditText) findViewById(R.id.numReqs_field);
        send = (Button) findViewById(R.id.send_button);
        stop = (Button) findViewById(R.id.stop_button);
        reset = (Button) findViewById(R.id.reset_button);
        numResp = (TextView) findViewById(R.id.num_response);
        meanTime = (TextView) findViewById(R.id.time_mean);
        medTime = (TextView) findViewById(R.id.time_med);

        editUrl = (EditText) findViewById(R.id.URL_field);
        TextWatcher sendWatcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {
            }

            @Override
            public void afterTextChanged(Editable editable) {
                String text = editable.toString();
                if (text.equals("")) {
                    send.setEnabled(false);
                } else {
                    send.setEnabled(true);
                }
            }
        };
        editUrl.addTextChangedListener(sendWatcher);

        mode_spinner = (Spinner) findViewById(R.id.mode_spinner);
        ArrayAdapter<CharSequence> adapter = ArrayAdapter.createFromResource(this,
                R.array.modes, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        mode_spinner.setAdapter(adapter);
        mode_spinner.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int position, long l) {
                mode = position;
                switch (position) {
                    case MANUAL :
                    case CONTINUOUS : editNumReqs.setVisibility(View.INVISIBLE);
                        break;
                    case FINITE : editNumReqs.setVisibility(View.VISIBLE);
                        break;
                    default:
                        Log.e("Spinner", "Unknown Mode selection: position=" + position);
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });

        series_spinner = (Spinner) findViewById(R.id.spinner_series);
        adapter = ArrayAdapter.createFromResource(this, R.array.spinner_series, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        series_spinner.setAdapter(adapter);

        xaxis_spinner = (Spinner) findViewById(R.id.spinner_xaxis);
        adapter = ArrayAdapter.createFromResource(this, R.array.spinner_xaxis, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        xaxis_spinner.setAdapter(adapter);

        yaxis_spinner = (Spinner) findViewById(R.id.spinner_yaxis);
        adapter = ArrayAdapter.createFromResource(this, R.array.spinner_yaxis, android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        yaxis_spinner.setAdapter(adapter);

        // Get Connection Objects
        teleMgr = (TelephonyManager) this.getSystemService(Context.TELEPHONY_SERVICE);
        connMgr = (ConnectivityManager) this.getSystemService(Context.CONNECTIVITY_SERVICE);

        // Handle Location API
        gApiClient = new GoogleApiClient.Builder(this)
            .enableAutoManage(this, this)
            .addApi(LocationServices.API)
            .addConnectionCallbacks(this)
            .addOnConnectionFailedListener(this)
            .build();
        locReq = LocationRequest.create()
                .setPriority(LocationRequest.PRIORITY_HIGH_ACCURACY)
                .setInterval(1000)        //   1 second, in milliseconds
                .setFastestInterval(250); // 1/4 second, in milliseconds

        //Set Defaults
        mode_spinner.setSelection(1); // Continuous
        editUrl.setText(prefs.getString(getString(R.string.pref_url), ""));
        series_spinner.setSelection(prefs.getInt(getString(R.string.pref_series), 0)); // Default to Network Class
        xaxis_spinner.setSelection(prefs.getInt(getString(R.string.pref_xaxis), 0));   // Default to ID
        yaxis_spinner.setSelection(prefs.getInt(getString(R.string.pref_yaxis), 1));   // Default to RTT

        // Create Chart
        chart = new Chart(this, (CombinedChart) findViewById(R.id.chart),
                stringToFieldType((String)xaxis_spinner.getSelectedItem()),
                stringToFieldType((String)yaxis_spinner.getSelectedItem()),
                stringToFieldType((String)series_spinner.getSelectedItem()));

        // Load Saved Data if there is any
        if (prefs.getBoolean(getString(R.string.pref_saved), false)) {
            try {
                File file = new File(getFilesDir(), getString(R.string.filename));
                ObjectInputStream ois = new ObjectInputStream(new FileInputStream(file));
                if (!chart.loadData(ois))
                    throw new Exception();
                ois.close();
            } catch (Exception e) {
                Toast.makeText(context, getString(R.string.load_err), Toast.LENGTH_SHORT).show();
                e.printStackTrace();
            }
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.toolbar, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_save:
                try {
                    File file = new File(getFilesDir(), getString(R.string.filename));
                    FileOutputStream fout = new FileOutputStream(file, false);
                    ObjectOutputStream oos = new ObjectOutputStream(fout);
                    Toast toast;
                    if (chart.writeData(oos)) {
                        toast = Toast.makeText(context, getString(R.string.save_toast), Toast.LENGTH_SHORT);
                        SharedPreferences.Editor edit = prefs.edit();
                        edit.putBoolean(getString(R.string.pref_saved), true);
                        edit.commit();
                    } else {
                        toast = Toast.makeText(context, getString(R.string.write_err), Toast.LENGTH_LONG);
                    }
                    toast.show();
                    oos.close();
                    fout.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    protected void onPause() {
        super.onPause();

        SharedPreferences.Editor editor = prefs.edit();
        editor.putString(getString(R.string.pref_url), editUrl.getText().toString());
        editor.apply();
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (gApiClient.isConnected())
            LocationServices.FusedLocationApi.removeLocationUpdates(gApiClient, this);
    }

    @Override
    public void onConnectionFailed(@NonNull ConnectionResult connectionResult) {
        /* Handle Google Play Services Connection Failure */
        // May need to add handling here if problem is encountered
        Toast.makeText(this, R.string.play_serv_error,Toast.LENGTH_LONG).show();
        acceptNoLocation = true;
        Log.w("onConnectionFailed", "acceptingNoLocation - Google Play, Err: " + connectionResult.getErrorMessage());
    }

    @Override
    public void onConnected(@Nullable Bundle bundle) {
        Log.i("onConnected", "Google Play - CONNECTION SUCCESSFUL");
    }

    @Override
    public void onConnectionSuspended(int i) {
        Log.i("onConnectionSuspended", "Reason: " + ((i == CAUSE_NETWORK_LOST) ? "Network Lost" : "Service Disconnected"));
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        if (requestCode == REQUEST_LOCATION ) {
            if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_DENIED) {
                acceptNoLocation = true;
                Log.w("onReqPermissResult", "Location Permission Denied, acceptingNoLocation = true");
            } else {
                Log.i("onReqPermissResult", "Location Permissions Granted");
            }
        }
    }

    @Override
    public void onLocationChanged(Location newLocation) {
        /* This application stores two previous locations in order to manually calculate speeds
           Such that the time interval between two points is neither too large nor too small. Once
           the prevMaxLocation is too old, it is replaced with the prevMinLocation, unless the
           prevMinLocation is itself too young

           Note: prevMinLocation not guaranteed to be > SPEED_MIN_INTERVAL. Must check before using.
                 prevMaxLocation not guaranteed to be < SPEED_MAX_INTERVAL. Must check before using.
        */

        // TODO Manual speed code commented out until I have time to debug - Rachel
        /* long prevMinInterval = -1;
        long prevMaxInterval = -1;
        long locationInterval = -1;

        if (newLocation != null && !newLocation.hasSpeed()) {
            Log.i("onLocationChanged", "no speed available, calculating prevLocations");
            prevMinInterval = (prevMinLocation != null) ? (newLocation.getTime() - prevMinLocation.getTime()) : -1;
            prevMaxInterval = (prevMaxLocation != null) ? (newLocation.getTime() - prevMaxLocation.getTime()) : -1;
            locationInterval = (location != null) ? (newLocation.getTime() - location.getTime()) : -1;
            Log.d("onLocationChanged", "locInt: " + locationInterval + " minInt: " + prevMinInterval + " maxInt: " + prevMaxInterval);

            // If need to update prevMin, update appropriately
            if (prevMinLocation == null) {
                prevMinLocation = newLocation;
            } else if (prevMinInterval > SPEED_MAX_INTERVAL) {
                if (location != null && locationInterval < SPEED_MAX_INTERVAL)
                    prevMinLocation = location;
                else
                    prevMinLocation = newLocation;
            }

            // If need to update prevMax, and prevMin can be used
            if ((prevMaxLocation == null || prevMaxInterval > SPEED_MAX_INTERVAL) &&
                    (prevMinInterval > SPEED_MIN_INTERVAL)) {
                Log.d("onLocChang", "max: " + prevMaxLocation + " min: " + prevMinLocation);
                prevMaxLocation = prevMinLocation;
                Log.d("onLocChang", "max: " + prevMaxLocation + " min: " + prevMinLocation);
                prevMinLocation = (location == null) ? newLocation : location;
                Log.d("onLocChang", "max: " + prevMaxLocation + " min: " + prevMinLocation + " --- ");
            }
        } */
        location = newLocation;

        // If we need to and can, calculate speed
        manSpeedCalc = false;
        /*if (location != null && !location.hasSpeed()) {
            if (prevMaxLocation != null && prevMaxInterval < SPEED_MAX_INTERVAL) {
                location.setSpeed(prevMaxLocation.distanceTo(location)/(location.getTime() - prevMaxLocation.getTime()));
                manSpeedCalc = true;
                Log.i("onLocationChanged", "Manual speed calc needed - Using previous. New speed: " + location.getSpeed());
            } else {
                Log.i("onLocationChanged", "Manual speed calc needed - Prev not usable");
            }
        } */
    }

    /* ABOVE - Android Functions
       BELOW - Application Specific Functions */

    public void onSend(View view) {
        // Check for and handle lack of connection / GPS
        if (confirmSend) {
            confirmSend = false;
        } else {
            NetworkInfo info = connMgr.getActiveNetworkInfo();
            if (info == null || !info.isConnectedOrConnecting()) {
                Toast.makeText(context, getString(R.string.no_connection), Toast.LENGTH_LONG).show();
                return;
            }
            // TODO Check if GPS enabled and prompt user to please turn it on?
        }
        // Disable controls
        send.setEnabled(false);
        reset.setEnabled(false);
        editUrl.setEnabled(false);
        mode_spinner.setEnabled(false);
        url = editUrl.getText().toString();

        if (type == HTTP && !(url.toLowerCase()).startsWith("http")) {
            url = "https://" + url;
        }

        else if (!URLUtil.isValidUrl(url)) {
            Toast.makeText(context, String.format(getString(R.string.invalid_URL), url), Toast.LENGTH_SHORT).show();
            editUrl.setEnabled(true);
            mode_spinner.setEnabled(true);
            send.setEnabled(true);
            reset.setEnabled(true);
            return;
        }

        if (mode == FINITE) {
            editNumReqs.setEnabled(false);
            numReqs = Integer.parseInt(editNumReqs.getText().toString());
        }

        // Send Request
        Log.i("URL", url);
        handleMode();
    }

    private void handleMode() {
        switch (mode) {
            case MANUAL :
                handleType();
                break;
            case CONTINUOUS :
                send.setVisibility(View.GONE);
                stop.setVisibility(View.VISIBLE);
                numReqs = 1;
                handleType();
                break;
            case FINITE :
                send.setVisibility(View.GONE);
                stop.setVisibility(View.VISIBLE);
                handleType();
                break;
            default :
                Log.e("handleMode", "Unknown Test Mode");
        }
    }

    private void handleType() {
        if (getLocation() == -1) {
            onRequestComplete(false, 0);
            return;
        }

        switch (type) {
            case HTTP :
                sendHTTP = new SendHTTP(this, url);
                timeSent = System.currentTimeMillis();
                netClass = getNetworkClass();
                carrier = teleMgr.getNetworkOperatorName();
                sendHTTP.sendRequest();
                break;
            default:
                Log.e("handleType", "Unknown Connection Type");
        }
    }

    void onRequestComplete(boolean success, long rtt) {
        boolean done;

        if (success) {
            DataPoint dataPoint = new DataPoint(type, url, rtt, timeSent, netClass, carrier,
                    location, uniqueID, manSpeedCalc);
            chart.addPoint(dataPoint);

            // Log Time
            times.add(rtt);
            sendDataPoint(dataPoint);
            numResp.setText(String.format(getString(R.string.num_times), times.size()));
            meanTime.setText(String.format(getString(R.string.mean_response), getMeanTime()));
            medTime.setText(String.format(getString(R.string.median_response), getMedTime()));

            // Handle Mode
            done = false;
            switch (mode) {
                case MANUAL :
                    done = true;
                    break;
                case CONTINUOUS :
                    if (numReqs == 0) {
                        done = true;
                    }
                    break;
                case FINITE :
                    numReqs--;
                    if (numReqs <= 0) {
                        done = true;
                    }
                    break;
                default :
                    Log.e("onRequestComplete", "Unknown mode");
            }
        } else {
            done = true;
        }

        // Unlock controls
        if (done) {
            switch (mode) {
                case MANUAL :
                    break;
                case CONTINUOUS :
                    stop.setVisibility(View.GONE);
                    send.setVisibility(View.VISIBLE);
                    break;
                case FINITE :
                    stop.setVisibility(View.GONE);
                    send.setVisibility(View.VISIBLE);
                    editNumReqs.setEnabled(true);
                    break;
                default :
                    Log.e("onRequestComplete", "Unknown mode");
            }
            editUrl.setEnabled(true);
            mode_spinner.setEnabled(true);
            send.setEnabled(true);
            reset.setEnabled(true);
            DataPoint.count = 0;
        } else {
            handleMode();
        }
    }

    public void onReset(View view) {
        times.clear();
        numResp.setText("");
        meanTime.setText("");
        medTime.setText("");
        editUrl.setEnabled(true);
        chart.reset();
    }

    public void onUpdate(View view) {
        chart.updateFields((String) xaxis_spinner.getSelectedItem(),
                           (String) yaxis_spinner.getSelectedItem(),
                           (String) series_spinner.getSelectedItem());

        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt(getString(R.string.pref_series), series_spinner.getSelectedItemPosition());
        editor.putInt(getString(R.string.pref_xaxis), xaxis_spinner.getSelectedItemPosition());
        editor.putInt(getString(R.string.pref_yaxis), yaxis_spinner.getSelectedItemPosition());
        editor.apply();
    }

    public void onStopReqs(View view) {
        numReqs = 0;
    }

    /* Get Network Class */
    /* Taken from: http://stackoverflow.com/questions/9283765/how-to-determine-if-network-type-is-2g-3g-or-4g
     *        and: http://stackoverflow.com/questions/17148371/check-whether-mobile-is-using-wifi-or-data-3g */
    public String getNetworkClass() {
        final android.net.NetworkInfo wifi = connMgr.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
        final android.net.NetworkInfo mobile = connMgr.getNetworkInfo(ConnectivityManager.TYPE_MOBILE);
        if (wifi.isConnectedOrConnecting ()) {
            return "WIFI";
        } else if (mobile.isConnectedOrConnecting ()) {
            int networkType = teleMgr.getNetworkType();
            switch (networkType) {
                case TelephonyManager.NETWORK_TYPE_GPRS:
                case TelephonyManager.NETWORK_TYPE_EDGE:
                case TelephonyManager.NETWORK_TYPE_CDMA:
                case TelephonyManager.NETWORK_TYPE_1xRTT:
                case TelephonyManager.NETWORK_TYPE_IDEN:
                    return "2G";
                case TelephonyManager.NETWORK_TYPE_UMTS:
                case TelephonyManager.NETWORK_TYPE_EVDO_0:
                case TelephonyManager.NETWORK_TYPE_EVDO_A:
                case TelephonyManager.NETWORK_TYPE_HSDPA:
                case TelephonyManager.NETWORK_TYPE_HSUPA:
                case TelephonyManager.NETWORK_TYPE_HSPA:
                case TelephonyManager.NETWORK_TYPE_EVDO_B:
                case TelephonyManager.NETWORK_TYPE_EHRPD:
                case TelephonyManager.NETWORK_TYPE_HSPAP:
                    return "3G";
                case TelephonyManager.NETWORK_TYPE_LTE:
                    return "4G";
                default:
                    Log.e("getNetworkClass", "Unknown Type: " + networkType);
                    return "Unknown";
            }
        } else {
            return "No Network";
        }
    }

    // 0 = Success, -1 = Failure
    private int getLocation() {
        if (!acceptNoLocation) {
            try {
                Location loc = LocationServices.FusedLocationApi.getLastLocation(gApiClient);
                if (loc == null) {
                    Log.i("getLocation", "Location null, requesting updates");
                    LocationServices.FusedLocationApi.requestLocationUpdates(gApiClient, locReq, this);
                } else {
                    onLocationChanged(loc);
                }
            } catch (SecurityException e) {
                ActivityCompat.requestPermissions(this,
                        new String[]{android.Manifest.permission.ACCESS_FINE_LOCATION},
                        REQUEST_LOCATION);
                location = null;
                return -1;
            } catch (IllegalStateException e) {
                Log.w("getLocation", "onStop probably called. Error: e.getMessage()");
                return -1;
            }
        } else {
            location = null;
        }
        return 0;
    }

    /* Statistic Functions */
    private float getMeanTime() {
        long sum = 0;
        for (long l : times) {
            sum += l;
        }
        if (times.size() == 0)
            return 0;
        else
            return sum / (float) times.size();
    }

    private float getMedTime() {
        Collections.sort(times);
        if (times.size() % 2 == 0) {
            return (times.get(times.size()/2) + times.get(times.size()/2 - 1)) / 2;
        } else {
            return times.get(times.size()/2);
        }
    }

    /* Helper Functions */
    String fieldTypeToString(DataPoint.Field f) {
        switch (f) {
            case TYPE : return getString(R.string.field_type);
            case URL : return getString(R.string.field_url);
            case RTT : return getString(R.string.field_rtt);
            case TIME: return getString(R.string.field_time);
            case ID : return getString(R.string.field_id);
            case NETCLASS : return getString(R.string.field_netClass);
            case SPEED : return getString(R.string.field_speed);
            default:
                Log.e("fieldTypeToString", "Unknown field");
                return "Unknown";
        }
    }

    DataPoint.Field stringToFieldType(String s) {
        if (s.equals(getString(R.string.field_type))) return DataPoint.Field.TYPE;
        if (s.equals(getString(R.string.field_url))) return DataPoint.Field.URL;
        if (s.equals(getString(R.string.field_rtt))) return DataPoint.Field.RTT;
        if (s.equals(getString(R.string.field_time))) return DataPoint.Field.TIME;
        if (s.equals(getString(R.string.field_id))) return DataPoint.Field.ID;
        if (s.equals(getString(R.string.field_netClass))) return DataPoint.Field.NETCLASS;
        if (s.equals(getString(R.string.field_speed))) return DataPoint.Field.SPEED;
        Log.e("stringToFieldType", "Failed to match string with Field");
        return null;
    }

    void sendDataPoint(DataPoint dataPoint) {
        Response.Listener<JSONObject> responseListener = new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {}
        };

        Response.ErrorListener errorListener = new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Log.e("sendDataPoint", error.toString());
            }
        };

        JsonObjectRequest request = new JsonObjectRequest(
                Request.Method.POST, SERVER_DATA_ENDPOINT, dataPoint.asJsonObject(),
                responseListener, errorListener);
        dataQueue.add(request);
    }
}
