package com.rkinney4.httptest;

import android.content.Context;
import android.util.Log;
import android.widget.Toast;

import com.android.volley.NetworkResponse;
import com.android.volley.Request;
import com.android.volley.RequestQueue;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.HttpHeaderParser;
import com.android.volley.toolbox.Volley;

/**
 * Created by Rachel on 2/24/2017.
 */

class SendHTTP implements Connection {

    private Context context;

    private static RequestQueue queue;
    private MetaRequest request;

    private long t1, t2;

    SendHTTP(Context context, String url) {
        this.context = context;
        if (queue == null)
            queue = Volley.newRequestQueue(context);
        setURL(url);
    }

    void setURL(String url) {
        request = new MetaRequest(Request.Method.HEAD, url,
                new Response.Listener<NetworkResponse>() {
                    @Override
                    // Called when HTTP Response is received
                    public void onResponse(NetworkResponse response) {
                        t2 = System.currentTimeMillis();
                        if (response.statusCode / 100 == 2) {
                            ((MainActivity) context).onRequestComplete(true, t2 - t1);
                        } else {
                            Log.e("HTTPResponseCode", response.statusCode + "");
                            ((MainActivity) context).onRequestComplete(false, -1);
                        }
                    }
                }, new Response.ErrorListener() {
                    @Override
                    // Called when HTTP Request results in an error
                    public void onErrorResponse(VolleyError error) {
                        try {
                            Toast toast = Toast.makeText(context,
                                    String.format(context.getString(R.string.HTTP_err),
                                            error.networkResponse.statusCode), Toast.LENGTH_LONG);
                            toast.show();
                        } catch (NullPointerException e)  {
                            Toast toast = Toast.makeText(context,
                                    error.getMessage(), Toast.LENGTH_LONG);
                            toast.show();
                        }
                        ((MainActivity) context).onRequestComplete(false, -1);
                    }
                });
    }

    public void sendRequest() {
        t1 = System.currentTimeMillis();
        queue.add(request);
    }

    /* Custom MetaRequest Class - Allows the full return of response information*/
    private class MetaRequest extends Request<NetworkResponse> {
        private final Response.Listener<NetworkResponse> mListener;

        MetaRequest(int method, String url, Response.Listener<NetworkResponse> listener,
                    Response.ErrorListener errorListener) {
            super(method, url, errorListener);
            mListener = listener;
        }

        @Override
        protected Response<NetworkResponse> parseNetworkResponse(NetworkResponse response) {
            return Response.success(response, HttpHeaderParser.parseCacheHeaders(response));
        }

        @Override
        protected void deliverResponse(NetworkResponse response) {
            mListener.onResponse(response);
        }
    }
}
