# Data Collection Server

## Usage

The server can be invoked (in debug mode) by running `./server.py`. By default,
the server listens on `127.0.0.1:8888` and saves the database in `data.sqlite`
in the CWD.

### Production Server

For production, you can specify the following environment variables to change
this default behavior:

  - `FLASK_HOST` & `FLASK_PORT` - host & port to bind to
  - `FLASK_DEBUG` - should be set to 0 (false)
  - `FLASK_DB` - the path to the sqlite DB file

The production server is run via a systemd service described in
`data-collection-server.service`. It is already installed in
`/usr/lib/systemd/system/data-collection-server.service` on `cloud2`. Should
you need to modify it, you must update this file and then
`service data-collection-server restart`.

This server runs on 8080 behind an nginx reverse proxy, which handles TLS
(with certs from letsencrypt).

The production server is stored at `/usr/local/src/data-collection-server`. It
is run as `data-collection-server`. The database is stored in
`/var/data-collection-server/db.sqlite` (which is owned by that user).

#### Deploying

To deploy to production make sure you have the git remote added:

```
# git remote add cloud ssh://abmpr42@cloud2.cnds.jhu.edu:8001/home/abmpr42/Connection-Test.git
```

Then you can deploy (the `master` branch) by simplying pushing:

```
# git push cloud master
```

Note you will need to restart the data collection server:

```
abmpr42@cloud2# sudo service data-collection-server restart
```

#### Maintenance

To view stdout/stderr from the server and see it's status use:

```
abmpr42@cloud2# sudo service data-collection-server status
```

## Database

You setup your database by running any migrations that you have yet to run.
Migrations are like Git commits for the schema of a database. They are numbered
so that you run them in the proper order. So to initially setup your DB, you
need to run:

```
$ sqlite3 data.sqlite < migrations/00_create_datapoints.sql
```

But later if another migration (for example, `01_add_carrier_name`) was added,
you would then need to run

```
$ sqlite3 data.sqlite < migrations/01_add_carrier_name.sql
```

before running the server to get your database schema up to date.

Note that for a production environment, you'll need to replace `data.sqlite`
with the path to the sqlite DB file.
